### Language
[简体中文](https://github.com/kszitt/react-native-sass-to-styleSheet/blob/master/README.md)

### Description
css file transform to react-native stylesheet  
1. support variable  
2. support for media queries  
3. support nesting  
4. support `transform`  
5. adapt to all kinds of mobile phones
6. support group selector

### Summary
- [Install](#install)
- [Use](#use)
  - [init](#init)
  - [start](#start)
  - [writing SCSS file](#writing-scss-file)
  - [after transform](#after-transform)
  - [use in react native](#use-in-react-native)
- [Examples](#examples)
  - [font](#font)
  - [margin, padding](#margin-padding)
  - [border](#border)
  - [textDecoration](#text-decoration)
  - [textShadow](#text-shadow)
  - [shadow](#shadow)
  - [transform](#transform)
  - [variable](#variable)
  - [group selector](#group-selector)
  - [media query](#media-query)
- [Automatic Template Generation](#automatic-template-generation)
  - [default automatic template generation](#default-automatic-template-generation)
  - [custom Automatic Template Generation](#custom-automatic-template-generation)


### Install
``` javascript
npm install react-native-sass-to-stylesheet --save-dev
```

### Use
##### init  
create`toStyles.js`, this content
``` javascript
const SassToStyles = require("react-native-sass-to-stylesheet");

SassToStyles.init(path[, options]);
```
.init(path[, options])  
- path{string} watch folder paths，request
- options{object}
    - space{number} indent value，default`2`
    - postfix{string} jS file suffix generated by transformation，default`Style.js` 
      for example:`home.scss => homeStyle.js`
    - initTransform{boolean} Whether to automatically convert all CSS files after starting the service, default`false`
    - adaptation{boolean} adapt to all kinds of mobile phones, default`true`. if a single style does not need to be adapted, Please add the ` !important`sign
    - templatePath{string} automatic conversion of file template path
##### start
``` javascript
node toStyles.js
```
##### writing SCSS file
in the path directory of `init ()`, create or upload`css or sass`file, save. generate `js`files in the current directory.  
`home.scss` file is as follows
``` scss
.header {
  font: 12px/24px;
  .logo {
    position: absolute;
    .img {
      width: 100px;
      height: 100px;
    }
  }
}
```
##### after transform
``` javascript
import {StyleSheet, PixelRatio, Dimensions} from 'react-native';
const pixelRatio = PixelRatio.get();
let {width, height} =  Dimensions.get('window');

function getAdaptation(num){    // You can customize this function in the options. template Path template
  let unitWidth = width / 1080; // 1080 => Width of UI design diagrams
  return parseFloat((num*unitWidth).toFixed(2));
}

let styles = {
  header: {
    fontSize: getAdaptation(12),
    lineHeight: getAdaptation(24)
  },
  header_logo: {
    position: "absolute"
  },
  header_logo_img: {
    width: getAdaptation(100),
    height: getAdaptation(100)
  }
};

const styleSheet = StyleSheet.create(styles);
export default styleSheet;
```
##### use in react native
``` javascript
import Style from "homeStyle.js";
...
render(){
    return (
        <View style={Style.header}>
            <View style={Style.header_logo}>
                <Image source={...} style={Style.header_logo_img}/>
            </View>
        </View>
    );
}
```

### Examples
##### font
``` scss
.main {
  font: italic bold 12px/24px "Arial";
  font-variant: small-caps, lining-nums;
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    fontVariant: [
      "small-caps",
      "lining-nums"
    ],
    fontSize: getAdaptation(12),
    lineHeight: getAdaptation(24),
    fontStyle: "italic",
    fontWeight: "bold",
    fontFamily: "Arial"
  }
};
```
##### margin padding
``` scss
.main {
  margin: 0 10px;
  padding: 1px 2px 3px;
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    marginTop: 0,
    marginBottom: 0,
    marginRight: getAdaptation(10),
    marginLeft: getAdaptation(10),
    paddingTop: getAdaptation(1),
    paddingBottom: getAdaptation(3),
    paddingRight: getAdaptation(2),
    paddingLeft: getAdaptation(2),
  }
};
```
##### border
``` scss
.main {
  border: 1px solid #333;
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    borderWidth: getAdaptation(1),
    borderColor: "#333",
    borderStyle: "solid"
  }
};
```
##### text-decoration
``` scss
.main {
  text-decoration: underline solid red;
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    textDecorationLine: "underline",
    textDecorationColor: "red",
    textDecorationStyle: "solid"
  }
};
```
##### text-shadow
``` scss
.main {
  text-shadow: 5px 5px 10px red;
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    textShadowOffset: {
      width: getAdaptation(5),
      height: getAdaptation(5)
    },
    textShadowRadio: getAdaptation(10),
    textShadowColor: "red"
  }
};
```
##### shadow
``` scss
.main {
  box-shadow: 10px 10px 5px .5 #888888;
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    shadowOffset: {
      width: getAdaptation(10),
      height: getAdaptation(10)
    },
    shadowRadio: getAdaptation(5),
    shadowOpacity: .5,
    shadowColor: "#888888"
  }
};
```
##### transform
``` scss
.main {
  transform: translate(10px, 20px) rotateY(-10.3deg) scaleX(.5) skew(60deg);
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    transform: [
      {
        translateX: getAdaptation(10),
        translateY: getAdaptation(20)
      },
      {
        rotateY: "-10.3deg"
      },
      {
        scaleX: .5
      },
      {
        skewX: "60deg"
      }
    ]
  }
};
```
##### variable
``` scss
$size: 12px !global; // other pages can also be used
$color: red;
.header {
  font: $size/24px;
  .left {
    color: $color;
  }
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  header: {
    fontSize: getAdaptation(12),
    lineHeight: getAdaptation(24)
  },
  header_left: {
    color: "red"
  }
};
```
##### group selector
``` scss
.main {
  display: flex;
  .left, .right {
    position: absolute;
    left: 0;
  }
  .left {
    left: 10px;
  }
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
let styles = {
  main: {
    display: "flex"
  },
  main_left: {
    position: "absolute",
    left: getAdaptation(10)
  },
  main_right: {
    position: "absolute",
    left: 0
  }
};
```
##### media query
``` scss
.main {
  width: 500px;
}
@media only screen and (min-width: 500px) and (max-width: 1000px) {
  .main {
    width: 100%;
    height: 1000px;
  }
}
```
↓ ↓ ↓ ↓ ↓ ↓
``` javascript
import {StyleSheet, PixelRatio, Dimensions} from 'react-native';
const pixelRatio = PixelRatio.get();
let {width, height} =  Dimensions.get('window');

function getAdaptation(num){    // You can customize this function in the options. template Path template
  let unitWidth = width / 1080; // 1080 => Width of UI design diagrams
  return parseFloat((num*unitWidth).toFixed(2));
}

let styles = {
  main: {
    width: getAdaptation(500)
  },
  main_top: {
    fontSize: getAdaptation(12)
  }
};


let media = {
  "width>=500&&width<=1000": {
    main: {
      width: "100%",
      height: getAdaptation(1000)
    }
  }
};

// 媒体查询
(function addMedia(){   // // You can customize this function in the options. template Path template
  for(let k in media){
    if(eval(k)){
      for(let j in media[k]){
        styles[j] = Object.assign(styles[j] || {}, media[k][j]);
      }
    }
  }
}());
const styleSheet = StyleSheet.create(styles);
export default styleSheet;
```

### Automatic Template Generation
##### default automatic template generation
``` javascript
import {StyleSheet, PixelRatio, Dimensions} from 'react-native';
const pixelRatio = PixelRatio.get();
let {width, height} =  Dimensions.get('window');

function getAdaptation(num){
  let unitWidth = width / 1080; // 1080 => Width of UI design diagrams
  return parseFloat((num*unitWidth).toFixed(2));
}

let styles = {};
let media = {};

const styleSheet = StyleSheet.create(styles);

export default styleSheet;
```
##### custom Automatic Template Generation
modify the `options.template` template path in `init(path [, options])`, use your template.
